#!/usr/bin/env python
# -*- coding: utf-8 -*-
import unittest
import time
from uuid import uuid4

from threading import Thread
from snorkel.task.TaskRunner import TaskRunner, TaskTerminatedError

__author__ = "Vemund Kvam"
__copyright__ = "Vemund Kvam"
__license__ = "none"


class TestTaskRunner(unittest.TestCase):

    abort_message = (uuid4(), uuid4(), uuid4())

    def test_success(self):
        runner = TaskRunner(TestTaskRunner.normal)
        a = runner.run(1)
        self.assertEqual(a, 1)

    def test_exception(self):
        runner = TaskRunner(TestTaskRunner.exception)
        with self.assertRaises(SomeException):
            runner.run(1)

    def test_terminator(self):
        runner = TaskRunner(TestTaskRunner.cpu_intensive)
        task_terminator = runner.build_terminator()

        with self.assertRaises(TaskTerminatedError) as e:

            def wait_send():
                nonlocal task_terminator
                time.sleep(.1)
                task_terminator.terminate(TestTaskRunner.abort_message)

            t = Thread(target=wait_send)
            t.daemon = True
            t.start()

            runner.run(1)
        self.assertEqual(e.exception.args[0], TestTaskRunner.abort_message)

    @staticmethod
    def normal(some):
        return some

    @staticmethod
    def exception(some):
        raise SomeException()

    @staticmethod
    def cpu_intensive(some):
        while True:
            pass


class SomeException(Exception):
    pass
